@@ -1,12 +1,17 @@
 # Repository: https://gitlab.com/quantify-os/quantify-core
 # Licensed according to the LICENCE file on the master branch
 """Module containing the types for use with the analysis classes"""
-from collections import OrderedDict
+from collections import UserDict
+
 from jsonschema import validate
+
 from quantify_core.utilities.general import load_json_schema
 
 
-class AnalysisSettings(OrderedDict):
+# pylint: disable=too-few-public-methods
+# WARNING! Do not inherit from dict! if you do, `AnalysisSettings.update will skip the
+# validation done in `__setitem__`.
+class AnalysisSettings(UserDict):
     """
     Analysis settings with built-in schema validations.
 
