@@ -1,10 +1,14 @@
+# pylint: disable=missing-module-docstring
+# pylint: disable=missing-class-docstring
+# pylint: disable=missing-function-docstring
+
 import pytest
 from jsonschema import ValidationError
 
 from quantify_core.analysis.types import AnalysisSettings
 
 
-def test_analysis_settings_valid():
+def test_analysis_settings_valid() -> None:
     _ = AnalysisSettings(
         {
             "mpl_dpi": 450,
@@ -16,7 +20,7 @@ def test_analysis_settings_valid():
     )
 
 
-def test_analysis_settings_invalid():
+def test_analysis_settings_invalid() -> None:
 
     with pytest.raises(ValidationError):
         _ = AnalysisSettings(
